use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
BNF
Base
Bindings
Calculator
Decoder
Durand
ESLIF
Encoder
Event
EventBitSet
Grammar
Interface
Introduction
JSON
Jean
Jeffrey
Kegler
Level
Logger
MarpaX
Properties
PropertyBitSet
Recognizer
RecognizerInterface
RegexCallout
Rule
String
Symbol
Tutorial
Type
Value
jeandamiendurand
jeffreykegler
lib
